/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  demoprogram for the C14 io driver

  -------------------------------------------------------------------------

                $RCSfile: C14Demo.c,v $
                
                $Author: K.Otto $
                
                $Revision: 1.2 $  $Date: 2004/09/20 11:16:32 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/10/28 -ct:   start of the implementation

****************************************************************************/

#include <XC161.h>
#include <intrins.h>
#include "generic\global.h"
#include "..\include\C14drv.h"
#include "..\include\timerdrv.h"


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define EEPROM_TYPE     EPRM25256
#define PROC_DATA_SIZE  0x100

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


WORD CRCPolynom (WORD wCRC_p, BYTE GENERIC* pData_p, WORD wSize_p);


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <main>                                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------
#define ADDR_HEX_CPU 0x3A8000

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------



//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

void main (void)
{
BYTE   bSwitchValue = 0;
BYTE   bChannel     = 0;
BYTE   bSwitchMask  = 0;
WORD   wVersion     = 0;
WORD   wDInput      = 0;
WORD   wADCValue    = 0;
DWORD  dwDInput     = 0;
DWORD  dwOldDInput  = 0;
BYTE   bOldHex      = 0;
BYTE   bOldDip      = 0;
WORD   i;
DWORD  dwStartTime;
BYTE   bTemp;
DWORD  dwDOut;
WORD   abAnaIn[4];

WORD   wCRC;
BYTE GENERIC* pnvSRAM=0x300000;

    PSW_IEN = 1;

    C14InitAsc0 (9600);

    //----------------------------------------------------------------
    // test function of EEPROM ...
    wCRC = CRCPolynom (0, pnvSRAM, 0x7FFE);

    printf("\nCRC = %04X", wCRC);

    if (wCRC != *(WORD GENERIC*)0x307FFE)
    {  printf(" --> Error on CRC-calculation of nvSRAM-CRC!!!!");
       printf("     (if it the first time of using this program, try again)");
    }   
    else
       printf(" --> CRC ok");

    // write new data
    for (i=0; i < 0x7FFE; i++)
    {
        pnvSRAM[i] = (BYTE)*(BYTE huge*)0x80000;
    }

    // CRC calculate and save
    *(WORD GENERIC*)0x307FFE = CRCPolynom (0, pnvSRAM, 0x7FFE);

    //----------------------------------------------------------------

    // Flash Device-ID read
    *(WORD xhuge*)0x600AAA = 0xAA;
    _nop_();
    *(WORD xhuge*)0x600554 = 0x55;
    _nop_();
    *(WORD xhuge*)0x600AAA = 0x90;
    _nop_();
    printf("\nMADE BY %04X, DEVICE-ID %04X", *(WORD xhuge*)0x600000, *(WORD xhuge*)0x600002);

    //----------------------------------------------------------------

    printf("\nSYSTEM-Status CPU:\n");
    printf("CPUCON1=%04X, CPUCON2=%04X, SYSCON0=%04X, SYSCON1=%04X, SYSCON3=%04X\n", 
             CPUCON1, CPUCON2, SYSCON0, SYSCON1, SYSCON3);
    printf("RSTCFG=%04X, SYSSTAT=%04X, PLLCON=%04X, FOCON=%04X, RSTCON=%04X\n", 
             RSTCFG, SYSSTAT, PLLCON, FOCON, RSTCON);
    printf("EBCMOD0=%04X, EBCMOD1=%04X\n", EBCMOD0, EBCMOD1);
    printf("TCONCS0=%04X, FCONCS0=%04X\n", TCONCS0, FCONCS0);
    printf("TCONCS1=%04X, FCONCS1=%04X, ADDRSEL1=%04X\n", TCONCS1, FCONCS1, ADDRSEL1);
    printf("TCONCS2=%04X, FCONCS2=%04X, ADDRSEL2=%04X\n", TCONCS2, FCONCS2, ADDRSEL2);
    printf("TCONCS3=%04X, FCONCS3=%04X, ADDRSEL3=%04X\n", TCONCS3, FCONCS3, ADDRSEL3);
    printf("TCONCS4=%04X, FCONCS4=%04X, ADDRSEL4=%04X\n", TCONCS4, FCONCS4, ADDRSEL4);
    printf("TCONCS7=%04X, FCONCS7=%04X, ADDRSEL7=%04X\n", TCONCS7, FCONCS7, ADDRSEL7);

    // init driver for IO's   
    printf ("\n\n\nInit IO driver ...");
    wVersion = C14Initialize ();
    StartTimer();

    printf ("V%1X.%02X", (wVersion / 0x100), (wVersion & 0x00FF));

    printf ("\nHardware: %02bX", C14GetIoHWCfg());

    C14SetErrLED (ON);

    while(C14GetSwitch() == SWITCH_RUN)
    {
        C14SetRunLED (ON);

        for (i=0; i < 20; i++)
        {
            dwStartTime = GetTickCount();
            // set outputs
            dwDOut = 1L <<i;
            C14SetOutGroup ((BYTE FAR*)&dwDOut, sizeof(dwDOut));
            while( (GetTickCount() - dwStartTime) < 300L);
            {
                // read inputs IN23..0 
                C14GetInGroup((BYTE FAR*)&dwDInput, sizeof(dwDInput));
                if( dwOldDInput != dwDInput )
                {
                  TRACE1(" 0x%08lX\n",dwDInput);
                  dwOldDInput = dwDInput;    
                }

                bTemp = C14GetHexNumber();
                if (bTemp != bOldHex)
                {
                   TRACE1("\nHex-Switch: %02BX", bTemp);
                   bOldHex = bTemp;
                }
                bTemp = C14GetDIPSwitch();
                if (bTemp != bOldDip)
                {
                   TRACE1("\nDIP-Switch: %02BX", bTemp);
                   bOldDip = bTemp;
                }

            }
//            printf("\nSYSTEM-State %04X, %02bX", SYSSTAT, pCFC[7]);
            C14ResetWatchdog();            
        }

    }

    C14SetRunLED (OFF);

    TRACE0("\n");
    TRACE1("\nHex-Switch: %02BX", C14GetHexNumber());
    TRACE1("\nDIP-Switch: %02BX", C14GetDIPSwitch());
    TRACE0("\n");


    bSwitchMask = 0;
    do
    {
        switch ( C14GetSwitch() )
        {
            case SWITCH_RUN:    //printf("\nRUN");
                                C14SetRunLED (ON);
                                C14SetErrLED (OFF);
                                bSwitchMask |= 0x01;
                                break;

            case SWITCH_STOP:   //printf("\nSTOP");
                                C14SetRunLED (OFF);
                                C14SetErrLED (OFF);
                                bSwitchMask |= 0x02;
                                break;

            case SWITCH_MRES:   //printf("\nMRES");
                                C14SetRunLED (OFF);
                                C14SetErrLED (ON);
                                bSwitchMask |= 0x04;
                                break;
        }
    }
    while (bSwitchMask != 0x07);

    C14SetRunLED (OFF);
    C14SetErrLED (OFF);


    if (C14GetSwitch() != SWITCH_RUN)
    {
       printf ("\nset RUN/STOP-Switch to RUN ...\n\n");
       while  (C14GetSwitch() != SWITCH_RUN);
    }

    C14SetRunLED (ON);

    do
    {
        // read inputs IN23..0
        C14GetInGroup((BYTE FAR*)&dwDInput, sizeof(dwDInput));

        if( dwOldDInput != dwDInput )
        {
            TRACE1(" 0x%08lX\n",dwDInput);
            dwOldDInput = dwDInput;

            // transistor-outputs OUT15..0 write (IN15..IN0)
            // relay-outputs REL3..0 write (mit IN19..16)
            C14SetOutGroup( (BYTE FAR*)&dwDInput, sizeof(dwDInput) );
        }
    }
    while (C14GetSwitch() == SWITCH_RUN);

    C14SetRunLED( OFF );

    TRACE0("\n");
    bSwitchValue = 0;
  
    do
    {
        bSwitchValue = C14GetHexNumber();
        TRACE1(" HEXswitch : 0x%02X\r",bSwitchValue);
    }
    while(bSwitchValue != 0xFF);

    TRACE0("\n");
    bSwitchValue = 0;

    C14SetErrLED( ON );

    do
    {
        bSwitchValue = C14GetDIPSwitch();
        TRACE1(" DIPswitch : 0x%02X\r",bSwitchValue);
    }
    while(bSwitchValue != 0xFF);
    
    C14SetRunLED( OFF );
    C14SetErrLED( OFF );

    TRACE0("\n read the 4 10Bit analog inputs\n");
    while(1)
    {
        TRACE0("\r");
        C14AdcGet ((BYTE FAR*) abAnaIn, sizeof(abAnaIn));

        for(bChannel = 0; bChannel < 4; bChannel++)
        {
            TRACE1(" 0x%04X",abAnaIn[bChannel]);
        }

    }


#pragma WARNING DISABLE = 177   // warning <unreachable code> = OFF
    TRACE0("\n It's enougth");
    while(1);
//#pragma WARNING ENABLE = 177    // warning <unreachable code> = ON
}

// EOF
// -------------------------------------------------------------------------
//
// Function:     CRCPolynom
//
// Description:  The function calculates the CRC using a formula 
//               The data bits are not reflected. That
//               means the bit 7 of a byte is the MSB-Bit. 
//               The start value of calculation must be NULL. The
//               bytes for saving CRC result must be regarded!! That
//               means the two bytes must preset with 0 and included
//               in calculation.
//
//               NOTICE: Do not use memory specifier for the
//                       local vars, because the compiler use register
//                       to store the local vars! The function is optimized
//                       for high speed (C167, 20MHz: 7.8ms/kByte). 
//
//               SOURCE: A.Suess
//
// Parameters:   wCRC_p                 - start value 
//               pData_p                - pointer to data 
//               wSize_p                - number of bytes
//
// 
// Returns:      CRC                    - checksum
//

#define POLYNOM 0x1021  // CCITT-Polynom X^16 + X^12 + X^5 + 1 (0x1021) (DIN 66 219)

WORD PUBLIC CRCPolynom (WORD wCRC_p, BYTE GENERIC* pData_p, WORD wSize_p)
{  
BYTE  j;
WORD  wHCRC;

    if (!wSize_p)
       return (wCRC_p);

    do
    {
        j = 8;

        do
        {
            // save carry
            wHCRC    = wCRC_p;
            wCRC_p <<= 1;

            if ( (wHCRC & 0x8000) != 0 )
            {
                wCRC_p ^= POLYNOM;
            }

            j--;

        } while (j > 0);

        wCRC_p ^= *pData_p;

        pData_p++;
        wSize_p--;

    } while (wSize_p > 0);

    return (wCRC_p);

}   



