/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  source file for eepromio driver

  -------------------------------------------------------------------------

                $RCSfile: EepromIo.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1.1.1 $  $Date: 2004/09/16 11:56:03 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2001/12/10 -ct:   start of the implemtation
  2003/10/28 -ct:   adjust for the XC161 CPU

****************************************************************************/

//---------------------------------------------------------------------------
// global includes
#include "generic/global.h"


//---------------------------------------------------------------------------
// modulspecific includes
#include "../include/spidrv.h"
#include "../include/eepromio.h"
#include "../include/eeprmdrv.h"


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------




/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <eeprom io driver>                                  */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------



//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     EepromInit
//
// Description:  This function initialize the eeprom driver
//
//
// Parameters:   bBaudrate_p : baudrate for the SSC0 interface
//
// 
// Returns:      none
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------
void EepromInit (BYTE bBaudrate_p)
{

    SpiInit((WORD)bBaudrate_p);

}

//---------------------------------------------------------------------------
//
// Function:     EepromWrBlk
//
// Description:  This function write a amount of bytes into the eeprom
//
//
// Parameters:   pbSrcPtr_p	: pointer to the source data
//               dwDstAdr_p	: destination address in the eeprom
//               dwSize_p	: amount of bytes to write
//               pEprom_p	: pointer to the eeprom descriptor
//
// Returns:      status
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------
BYTE EepromWrBlk (BYTE FAR* pbSrcPtr_p, DWORD dwDstAdr_p, 
                  DWORD dwSize_p, tEprm FAR* pEprom_p)
{
BYTE    bEepromNr;
DWORD   dwDstAdr;
DWORD dwWriteDataSize;
DWORD dwPageOffset;

    do
    {
        // get the next page border
        dwPageOffset = ((dwDstAdr_p + pEprom_p->m_bPageSize) & ~(pEprom_p->m_bPageSize - 1));
        
        // number of bytes to the next page border
        dwWriteDataSize = dwPageOffset - dwDstAdr_p;
        
        // If write less data than the dwWriteDataSize are, then write all the data. 
        // Otherwise write data to the next border and then always page sizes
        if(dwWriteDataSize > dwSize_p)
        {
            dwWriteDataSize = dwSize_p;
        }
        
        // select the eeprom
        if(dwDstAdr_p > pEprom_p->m_dwEndAdr)
        {
            // Nur ein EEPROM parametriert?
            if (pEprom_p->m_bAmount == 0)
               // Fehler: ungltiger Zugriff
               return (~EEPROM_SUCCESSFUL);

            dwDstAdr = dwDstAdr_p - pEprom_p->m_dwEndAdr - 1;
            bEepromNr = 2;
        }
        else
        {
            dwDstAdr = dwDstAdr_p;
            bEepromNr = 1;
        }
        
        SpiWritePage(pbSrcPtr_p, dwDstAdr, dwWriteDataSize, bEepromNr);
        
        dwSize_p    -= dwWriteDataSize;
        dwDstAdr_p	+= dwWriteDataSize;
        pbSrcPtr_p	+= dwWriteDataSize;
    
    }
    while(dwSize_p != 0);
    
    return(EEPROM_SUCCESSFUL);
}

//---------------------------------------------------------------------------
//
// Function:     EepromRdBlk
//
// Description:  This function read a amount of bytes out of the eeprom
//
// Parameters:   pbDstPtr_p	: pointer to the destination area in RAM
//               dwSrcAdr_p : source address in the eeprom
//         		 dwSize_p	: amount of data
//			     pEprom_p	: pointer to the eeprom descriptor
// 
// Returns:      none
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------
BYTE EepromRdBlk (BYTE FAR* pbDstPtr_p, DWORD dwSrcAdr_p, 
                  DWORD dwSize_p, tEprm FAR* pEprom_p)
{
BYTE   bEepromNr;
DWORD  dwReadDataSize;
DWORD  dwSrcAdr;

    do
    {
        if(dwSrcAdr_p > pEprom_p->m_dwEndAdr)
        {
            // Nur ein EEPROM parametriert?
            if (pEprom_p->m_bAmount == 0)
               // Fehler: ungltiger Zugriff
               return (~EEPROM_SUCCESSFUL);
            // data lie in the second eeprom
            dwSrcAdr = dwSrcAdr_p - pEprom_p->m_dwEndAdr - 1;
            bEepromNr = 2;
        }
        else
        {
            // data lie in the first eeprom
            dwSrcAdr = dwSrcAdr_p;
            bEepromNr = 1;
        }
        
        if((dwSrcAdr_p + dwSize_p - 1) > pEprom_p->m_dwEndAdr)
        {
            if(bEepromNr == 1)
            {
               // read data until the end of the first eeprom
               dwReadDataSize = pEprom_p->m_dwEndAdr - dwSrcAdr_p + 1;
            }
            else
            {
                // read data until the end of the second eeprom
                dwReadDataSize = dwSize_p;
            }
        }
        else
        {
            // read all data
            dwReadDataSize = dwSize_p;
        }
        
        // read data
        SpiReadPage(pbDstPtr_p, dwSrcAdr, dwReadDataSize, bEepromNr);
        
        dwSize_p		-= dwReadDataSize;
        dwSrcAdr_p	+= dwReadDataSize;
        pbDstPtr_p	+= dwReadDataSize;
    }
    while(dwSize_p != 0);
    
    return(EEPROM_SUCCESSFUL);
}

//---------------------------------------------------------------------------
//
// Function:     EEPGetBaudrate
//
// Description:  This function deliver the actual baudrate of the eeprom
//
// Parameters:   bDeviceType_p : eeprom typ
//
// Returns:      Baudrate
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------
BYTE EEPGetBaudrate (BYTE bDeviceType_p)
{
    switch (bDeviceType_p)
    {
        case EPRM25080 :  return (0);
        case EPRM25160 :  return (0);
        case EPRM25170 :  return (0);
        case EPRM25640 :  return (0);
        case EPRM25650 :  return (0);
        case EPRM25128 :  return (0);
        case EPRM25256 :  return (BR2M);
        case EPRM25512 :  return (0);
        default        :  return (0);
    }

}

//---------------------------------------------------------------------------
//
// Function:     EEPGetSize
//
// Description:  This function deliver the memory size of the eeprom
//
// Parameters:   bDeviceType_p : eeprom typ
//
// Returns:      memory size
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------
DWORD  EEPGetSize (BYTE bDeviceType_p)
{
    switch (bDeviceType_p)
    {
        case EPRM25080 :  return (0);		        //  1 kByte
        case EPRM25160 :  return (0);   	        //  2 kByte
        case EPRM25170 :  return (0);   	        //  2 kByte
        case EPRM25640 :  return (0);		        //  8 kByte
        case EPRM25650 :  return (0);		        //  8 kByte
        case EPRM25128 :  return (0);	            // 16 kByte
        case EPRM25256 :  return (32768);	        // 32 kByte
        case EPRM25512 :  return (0);	            // 64 kByte
        default:          return (0);
    }
}

//---------------------------------------------------------------------------
//
// Function:     EEPGetPageSize
//
// Description:  This function deliver the page size of the eeprom
//
// Parameters:   bDeviceType_p : eeprom typ
//
// Returns:      page size
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------
BYTE EEPGetPageSize (BYTE bDeviceType_p)
{
    switch (bDeviceType_p)
    {
        case EPRM25080 :  return (0);		        //  16 Byte Page
        case EPRM25160 :  return (0);   	        //  16 Byte Page
        case EPRM25170 :  return (0);   	        //  32 Byte Page
        case EPRM25640 :  return (0);		        //  32 Byte Page
        case EPRM25650 :  return (0);		        //  32 Byte Page
        case EPRM25128 :  return (0);		        //  64 Byte Page
        case EPRM25256 :  return (64);		        //  64 Byte Page
        case EPRM25512 :  return (0);		        // 128Byte Page
        default:          return (0);
    }

}

//---------------------------------------------------------------------------
//
// Function:     EEPGetEndAdr
//
// Description:  This function deliver the max. memory address of the 
//               mounted eeprom.
//
// Parameters:   bDeviceType_p : eeprom typ
//
// Returns:      max. memory address
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------
DWORD EEPGetEndAdr (BYTE bDeviceType_p)
{
    return (EEPGetSize(bDeviceType_p) - 1);
}

// EOF
