/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  source file for SSC0 unit of the XC161 controller

  -------------------------------------------------------------------------

                $RCSfile: SpiDrv.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1.1.1 $  $Date: 2004/09/16 11:56:03 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/10/28 -ct:   start of the implementation version 1.00

****************************************************************************/

#include <stdio.h>

#include "generic/global.h"
#include "generic/gmacros.h"
#include "../include/C14drv.h"


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define DOMAIN_SPI      0x01
#define REVISION_SPI    0x00

// SPI eeprom command 
#define CMD_WRSR  0x1             // Write status register
#define CMD_WRITE 0x2             // Write data to memeory array beginning at 
                                  // selected adress
#define CMD_READ  0x3             // Read data from memeory array beginning at
                                  // selected adress
#define CMD_WRDI  0x4             // Reset the write enable latch 
                                  // (disable write operations)
#define CMD_RDSR  0x5             // Read status register
#define CMD_WREN  0x6             // Set the write enable latch
                                  // (enable write operations)
#define CMD_DUMMY 0xFF            // DUMMY byte
#define WIPBIT    0x0             // Statuswert vom WIP-Bit
  
//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

#ifndef _tWORD

#define _tWORD

    typedef union
    {
      WORD w;
      struct
        {
        BYTE low;
        BYTE high;
        } b;
    
    }tWORD;

#endif    


//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

static void SpiWren   (BYTE bEPROMNR_p);
static void SpiWrdi   (BYTE bEPROMNR_p);
static BYTE SpiRdsr   (BYTE bEPROMNR_p);
static void SpiWipPoll(BYTE bEPROMNR_p);

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <driver for SSC0>                                   */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Functions for the access to the internal SSC0 iinterface of the XC161
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

static BYTE huge abCommand_l[128+3];

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------



//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     SpiInit
//
// Description:  This fuction initializes the internal SSC0 interface
//
//
// Parameters:   baudrate for the SSC0 communication
//
// 
// Returns:      version of the driver
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------
WORD PUBLIC SpiInit(WORD wBaudrate_p)
{
WORD wTmp;

    wTmp = DOMAIN_SPI;
    wTmp <<= 8;
    wTmp |= REVISION_SPI;
    
    return(wTmp); 
}

//---------------------------------------------------------------------------
//
// Function:     SpiReadPage
//
// Description:  This fuction reads data out of the spi device
//
//
// Parameters:   pbDstPtr_p : pointer for the destination buffer in the RAM
//               dwSrcAdr_p : source address in the spi device
//               dwSize_p   : count of data to read
//               bEpromNr_p : number of the spi device
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------
void PUBLIC SpiReadPage (BYTE FAR* pbDstPtr_p, DWORD dwSrcAdr_p, DWORD dwSize_p, BYTE bEpromNr_p)
{
DWORD dwIndex;
BYTE  abCommand[4];

/*
    TRACE1("\nSpiReadPage -> pbDstPtr_p : 0x%08X",(BYTE FAR*)pbDstPtr_p);
    TRACE1("\nSpiReadPage -> dwSrcAdr_p : 0x%08X",dwSrcAdr_p);
    TRACE1("\nSpiReadPage -> dwSize_p   : 0x%08X",dwSize_p);
    TRACE1("\nSpiReadPage -> bEpromNr_p : 0x%08X",bEpromNr_p);
*/
    //------------------------------------------------------------------
    // Ressource parametrieren: Sendepuffer, Empfangspuffer deklarieren
    // Mu nur einmal ausgefhrt werden, da sich die Parameter nicht ndern.
    C14SpiInitRessource (SPI_DEVICE_EEPROM, 4, abCommand, abCommand, NULL);
    
    for (dwIndex = 0; dwIndex < dwSize_p; dwIndex++)
    {        
        abCommand[0] = CMD_READ;  
        abCommand[1] = (BYTE)(dwSrcAdr_p>>8);  
        abCommand[2] = (BYTE)dwSrcAdr_p;
        abCommand[3] = CMD_DUMMY;

        C14SpiStartSingleCycle(SPI_DEVICE_EEPROM);

        while (C14SpiGetRrcState(SPI_DEVICE_EEPROM) == SPI_DEVICE_BUSY);

        // TRACE2("\nSpiReadPage -> Daten 0x%02X auf Adresse 0x%04X", abCommand[3], (WORD)dwSrcAdr_p);   
        
        pbDstPtr_p[dwIndex] = abCommand[3];                 // read the next byte
        
        dwSrcAdr_p++;                                       // increment the source adress
    }
}

//---------------------------------------------------------------------------
//
// Function:     SpiWritePage
//
// Description:  This fuction writes data into the spi device. The size of
//               data can max. increase to the size of one eeprom page size.
//
// Parameters:   pbSrcPtr_p : pointer to the source data in RAM
//               dwDstAdr_p : destination address in the spi device
//               dwSize_p   : count of data to write
//               bEpromNr_p : number of the spi device
//
// Returns:      version of the driver
//
//
// State:        
//
//---------------------------------------------------------------------------
void PUBLIC SpiWritePage(BYTE FAR* pbSrcPtr_p, DWORD dwDstAdr_p, DWORD dwSize_p, BYTE bEpromNr_p)
{
WORD  wIndex;

/* 
    TRACE1("\nSpiWritePage -> pbSrcPtr_p : 0x%08X",(BYTE FAR*)pbSrcPtr_p);
    TRACE1("\nSpiWritePage -> dwDstAdr_p : 0x%08X",dwDstAdr_p);
    TRACE1("\nSpiWritePage -> dwSize_p   : 0x%08X",dwSize_p);
    TRACE1("\nSpiWritePage -> bEpromNr_p : 0x%08X",bEpromNr_p);
*/
    //------------------------------------------------------------------
    // set the write enable bit
    SpiWren(SPI_DEVICE_EEPROM);

    //------------------------------------------------------------------
    abCommand_l[0] = CMD_WRITE;  
    abCommand_l[1] = (BYTE)(dwDstAdr_p>>8);  
    abCommand_l[2] = (BYTE)dwDstAdr_p;

    for (wIndex = 0; wIndex < dwSize_p; wIndex++)
        // Anwenderdaten in Sendepuffer kopieren (Sendepuffer mu innerhalb 
        // eines Segmentes liegen, Anwenderdaten liegen irgendwo)
        abCommand_l[3 + wIndex] = pbSrcPtr_p[wIndex];

    // Ressource parametrieren: Sendepuffer, Empfangspuffer deklarieren
    C14SpiInitRessource (SPI_DEVICE_EEPROM, (WORD)dwSize_p+3, abCommand_l, abCommand_l, NULL);
    // Transfer starten
    C14SpiStartSingleCycle(SPI_DEVICE_EEPROM);

    // Status pollen: bertragung abgeschlossen?
    while (C14SpiGetRrcState(SPI_DEVICE_EEPROM) == SPI_DEVICE_BUSY);
    
    //------------------------------------------------------------------
    // check if "write in process" is to end    
    SpiWipPoll(SPI_DEVICE_EEPROM);                    
    // clear the write enable bit 
    SpiWrdi(SPI_DEVICE_EEPROM);                       
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     SpiWren
//
// Description:  This function sets the "write enable" bit in the status
//               register.
//
// Parameters:   bEPROMNR_p : number of the spi device
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------
static void SpiWren(BYTE bEPROMNR_p) 
{
BYTE abCommand[] = {CMD_WREN};

    C14SpiInitRessource (bEPROMNR_p, tabentries (abCommand), 
                         abCommand, abCommand, NULL);

    C14SpiStartSingleCycle(bEPROMNR_p);

    while (C14SpiGetRrcState(bEPROMNR_p) == SPI_DEVICE_BUSY);
}

//---------------------------------------------------------------------------
//
// Function:     SpiWrdi
//
// Description:  This function clears the "write enable" bit in the status
//               register.
//
// Parameters:   bEPROMNR_p : number of the spi device
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------
static void SpiWrdi(BYTE bEPROMNR_p)
{
BYTE abCommand[] = {CMD_WRDI};

    C14SpiInitRessource (bEPROMNR_p, tabentries (abCommand), 
                         abCommand, abCommand, NULL);

    C14SpiStartSingleCycle(bEPROMNR_p);

    while (C14SpiGetRrcState(bEPROMNR_p) == SPI_DEVICE_BUSY);
}

//---------------------------------------------------------------------------
//
// Function:     SpiRdsr
//
// Description:  This function reads the status register of the spi device
//
// Parameters:   bEPROMNR_p : number of the spi device
//
// Returns:      Value of the status register
//
// State:        
//
//---------------------------------------------------------------------------
static BYTE SpiRdsr(BYTE bEPROMNR_p) 
{
BYTE abCommand[] = {CMD_RDSR, 0xFF};

    C14SpiInitRessource (bEPROMNR_p, tabentries (abCommand), 
                         abCommand, abCommand, NULL);
    C14SpiStartSingleCycle(bEPROMNR_p);

    while (C14SpiGetRrcState(bEPROMNR_p) == SPI_DEVICE_BUSY);

    return (abCommand[1]);
}

//---------------------------------------------------------------------------
//
// Function:     SpiWipPoll
//
// Description:  This function checks, if a write process in the eeprom is
//               ended.
//
// Parameters:   bEPROMNR_p : number of the spi device
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------
static void SpiWipPoll(BYTE bEPROMNR_p)
{
BYTE bTmp;

    do 
    {
        bTmp = SpiRdsr(bEPROMNR_p);             // read the status register
        bTmp &= 0x01;                           // mask the WIP bit
    } 
    while (bTmp != WIPBIT);                     // repeat until WIP bit is "0"
}

// EOF