/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  Sourcefile for the system timer driver for xc161 controller

  -------------------------------------------------------------------------

                $RCSfile: TimerDrv.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1.1.1 $  $Date: 2004/09/16 11:56:05 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/10/27 -ct:   start of the implementation version 1.00

****************************************************************************/

#include <xc161.h>
#include <intrins.h>

#include "generic/global.h"
#include "../include/timerdrv.h"

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define DOMAIN              0x01            // Versions-Nummer 1.00
#define REVISION            0x00

#define INT_ENABLE          0x0040
#define PRIORITY_TASK_TMR   0x0030         // interrupt priority of system timer

#define TICKS_PER_MILLISEC  5000           // 1ms = 5000 * 200ns

#define TASK_SYSTEM_TMR     0x10           // Task-Number CC0

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

tTime TimerValue_g;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------




/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <system timer>                                      */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
//
// In this class are all functions for the system timer implemented.
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------



//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     StartTimer
//
// Description:  This function initialize the base timer 0 as system timer
//               for using with CAPCOM1-Unit. CC0 is used for generating
//               the system clock of 1ms.
//
// Parameters:   none
//
// 
// Returns:      versionnumber of the timer driver
//
//
// State:        tested (06.04.2004 A.S.)
//
//---------------------------------------------------------------------------
WORD PUBLIC StartTimer   (void)
{
WORD wTmp;

    TimerValue_g = 0; // init tick count

    //-----------------------------------------------------------------------
    //  Configuration of CAPCOM1 Timer 0:
    //  - timer 0 works in timer mode
    //  - prescaler factor is 8 (staggered mode)
    //  - input frequency is 5MHz (40MHz CPU-Takt)
    //  - timer run bit is set
    CC1_T01CON    &=  0xFF00;
    CC1_T0         =  0x0000;
    CC1_T0IC       =  0x0000;
    CC1_T01CON    |=  0x0040;

    //-----------------------------------------------------------------------
    //  Configuration of CAPCOM1 channel 0 for system timer:
    CC1_CC0IC      =  0x0000;      // disable interrupt
    CC1_CC0        =  CC1_T0 + TICKS_PER_MILLISEC;
    CC1_M0        &=  0xFFF0;      // clear mode field
    CC1_M0        |=  0x0004;      // set compare mode 0
    CC1_CC0IC      =  INT_ENABLE + PRIORITY_TASK_TMR;

    //-----------------------------------------------------------------------
    wTmp = DOMAIN;
    wTmp <<= 8;
    wTmp |= REVISION;
    
    return(wTmp); 
}

//---------------------------------------------------------------------------
//
// Function:     DisableTimer
//
// Description:  This function stop the system timer
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (06.04.2004 A.S.)
//
//---------------------------------------------------------------------------
void PUBLIC DisableTimer (void)
{
    CC1_M0        &=  0xFFF0;      // clear mode field
    CC1_CC0IC      =  0x0000;      // disable interrupt
}

//---------------------------------------------------------------------------
//
// Function:     GetTickCount
//
// Description:  This function get the actual timer tick count of the system
//               timer.
//
// Parameters:   none
//
// 
// Returns:      timer tick count in ms
//
//
// State:        tested (06.04.2004 A.S.)
//
//---------------------------------------------------------------------------
DWORD PUBLIC GetTickCount (void)
{
tTime dwTimerTicks;

    // disable timerinterrupt
    _atomic_(0);
    dwTimerTicks = TimerValue_g;
    _endatomic_();

    return (dwTimerTicks);
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     TimerIsr
//
// Description:  This function is CC0 interrupt service routine.
//               Here the internal timer tick count variable is incremented
//               every ms. 
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (06.04.2004 A.S.)
//
//---------------------------------------------------------------------------

static void TimerIsr (void) interrupt TASK_SYSTEM_TMR
{
    #ifdef TEST_IM_MONITOR
       // So funktioniert es im Monitor-Mode
       CC1_CC0 =  CC1_T0 + TICKS_PER_MILLISEC;
    #else
      // So ist es genauer, fhrt zu berlufen im Monitor-Mode,
      // wenn Applikation am Breakpoint.wartet.
       CC1_CC0 += TICKS_PER_MILLISEC;
    #endif
  
    TimerValue_g += 1;
}



// EOF

